@@ -1,14 +1,13 @@
 use crate::{
     buffers::{BufferDescription, BufferKind},
     execute::{execute_columns, execute_tables, execute_with_parameters},
-    handles::{self, State, Statement, StatementImpl},
+    handles::{self, slice_to_utf8, SqlText, State, Statement, StatementImpl},
     parameter_collection::ParameterRefCollection,
     statement_connection::StatementConnection,
     CursorImpl, Error, Preallocated, Prepared,
 };
 use odbc_sys::HDbc;
 use std::{borrow::Cow, mem::ManuallyDrop, str, thread::panicking};
-use widestring::{U16Str, U16String};
 
 impl<'conn> Drop for Connection<'conn> {
     fn drop(&mut self) {
@@ -70,7 +69,7 @@ impl<'c> Connection<'c> {
     /// Transfer ownership of this open connection to a wrapper around the raw ODBC pointer. The
     /// wrapper allows you to call ODBC functions on the handle, but doesn't care if the connection
     /// is in the right state.
-    /// 
+    ///
     /// You should not have a need to call this method if your usecase is covered by this library,
     /// but, in case it is not, this may help you to break out of the type structure which might be
     /// to rigid for you, while simultaniously abondoning its safeguards.
@@ -78,16 +77,6 @@ impl<'c> Connection<'c> {
         unsafe { handles::Connection::new(ManuallyDrop::new(self).connection.as_sys()) }
     }
 
-    /// Executes an sql statement using a wide string. See [`Self::execute`].
-    pub fn execute_utf16(
-        &self,
-        query: &U16Str,
-        params: impl ParameterRefCollection,
-    ) -> Result<Option<CursorImpl<StatementImpl<'_>>>, Error> {
-        let lazy_statement = move || self.allocate_statement();
-        execute_with_parameters(lazy_statement, Some(query), params)
-    }
-
     /// Executes an SQL statement. This is the fastest way to submit an SQL statement for one-time
     /// execution.
     ///
@@ -122,8 +111,9 @@ impl<'c> Connection<'c> {
         query: &str,
         params: impl ParameterRefCollection,
     ) -> Result<Option<CursorImpl<StatementImpl<'_>>>, Error> {
-        let query = U16String::from_str(query);
-        self.execute_utf16(&query, params)
+        let query = SqlText::new(query);
+        let lazy_statement = move || self.allocate_statement();
+        execute_with_parameters(lazy_statement, Some(&query), params)
     }
 
     /// In some use cases there you only execute a single statement, or the time to open a
@@ -172,19 +162,6 @@ impl<'c> Connection<'c> {
         Ok(Some(cursor))
     }
 
-    /// Prepares an SQL statement. This is recommended for repeated execution of similar queries.
-    ///
-    /// # Parameters
-    ///
-    /// * `query`: The text representation of the SQL statement. E.g. "SELECT * FROM my_table;". `?`
-    ///   may be used as a placeholder in the statement text, to be replaced with parameters during
-    ///   execution.
-    pub fn prepare_utf16(&self, query: &U16Str) -> Result<Prepared<'_>, Error> {
-        let mut stmt = self.allocate_statement()?;
-        stmt.prepare(query).into_result(&stmt)?;
-        Ok(Prepared::new(stmt))
-    }
-
     /// Prepares an SQL statement. This is recommended for repeated execution of similar queries.
     ///
     /// # Parameters
@@ -193,8 +170,10 @@ impl<'c> Connection<'c> {
     ///   may be used as a placeholder in the statement text, to be replaced with parameters during
     ///   execution.
     pub fn prepare(&self, query: &str) -> Result<Prepared<'_>, Error> {
-        let query = U16String::from_str(query);
-        self.prepare_utf16(&query)
+        let query = SqlText::new(query);
+        let mut stmt = self.allocate_statement()?;
+        stmt.prepare(&query).into_result(&stmt)?;
+        Ok(Prepared::new(stmt))
     }
 
     /// Allocates an SQL statement handle. This is recommended if you want to sequentially execute
@@ -319,20 +298,14 @@ impl<'c> Connection<'c> {
         force_send_sync::Send::new(self)
     }
 
-    /// Fetch the name of the database management system used by the connection and store it into
-    /// the provided `buf`.
-    pub fn fetch_database_management_system_name(&self, buf: &mut Vec<u16>) -> Result<(), Error> {
-        self.connection
-            .fetch_database_management_system_name(buf)
-            .into_result(&self.connection)
-    }
-
     /// Get the name of the database management system used by the connection.
     pub fn database_management_system_name(&self) -> Result<String, Error> {
         let mut buf = Vec::new();
-        self.fetch_database_management_system_name(&mut buf)?;
-        let name = U16String::from_vec(buf);
-        Ok(name.to_string().unwrap())
+        self.connection
+            .fetch_database_management_system_name(&mut buf)
+            .into_result(&self.connection)?;
+        let name = slice_to_utf8(&buf).unwrap();
+        Ok(name)
     }
 
     /// Maximum length of catalog names.
@@ -363,20 +336,14 @@ impl<'c> Connection<'c> {
             .into_result(&self.connection)
     }
 
-    /// Fetch the name of the current catalog being used by the connection and store it into the
-    /// provided `buf`.
-    pub fn fetch_current_catalog(&self, buf: &mut Vec<u16>) -> Result<(), Error> {
-        self.connection
-            .fetch_current_catalog(buf)
-            .into_result(&self.connection)
-    }
-
     /// Get the name of the current catalog being used by the connection.
     pub fn current_catalog(&self) -> Result<String, Error> {
         let mut buf = Vec::new();
-        self.fetch_current_catalog(&mut buf)?;
-        let name = U16String::from_vec(buf);
-        Ok(name.to_string().unwrap())
+        self.connection
+            .fetch_current_catalog(&mut buf)
+            .into_result(&self.connection)?;
+        let name = slice_to_utf8(&buf).expect("Return catalog must be correctly encoded");
+        Ok(name)
     }
 
     /// A cursor describing columns of all tables matching the patterns. Patterns support as
@@ -397,10 +364,10 @@ impl<'c> Connection<'c> {
     ) -> Result<CursorImpl<StatementImpl<'_>>, Error> {
         execute_columns(
             self.allocate_statement()?,
-            &U16String::from_str(catalog_name),
-            &U16String::from_str(schema_name),
-            &U16String::from_str(table_name),
-            &U16String::from_str(column_name),
+            &SqlText::new(catalog_name),
+            &SqlText::new(schema_name),
+            &SqlText::new(table_name),
+            &SqlText::new(column_name),
         )
     }
 
@@ -424,8 +391,8 @@ impl<'c> Connection<'c> {
     /// use odbc_api::{Connection, Cursor, Error, ResultSetMetadata, buffers::TextRowSet};
     ///
     /// fn print_all_tables(conn: &Connection<'_>) -> Result<(), Error> {
-    ///     // Set all filters to None, to really print all tables
-    ///     let cursor = conn.tables(None, None, None, None)?;
+    ///     // Set all filters to an empty string, to really print all tables
+    ///     let cursor = conn.tables("", "", "", "")?;
     ///
     ///     // The column are gonna be TABLE_CAT,TABLE_SCHEM,TABLE_NAME,TABLE_TYPE,REMARKS, but may
     ///     // also contain additional driver specific columns.
@@ -463,23 +430,19 @@ impl<'c> Connection<'c> {
     /// ```
     pub fn tables(
         &self,
-        catalog_name: Option<&str>,
-        schema_name: Option<&str>,
-        table_name: Option<&str>,
-        table_type: Option<&str>,
+        catalog_name: &str,
+        schema_name: &str,
+        table_name: &str,
+        table_type: &str,
     ) -> Result<CursorImpl<StatementImpl<'_>>, Error> {
         let statement = self.allocate_statement()?;
 
-        let catalog_name = catalog_name.map(U16String::from_str);
-        let schema_name = schema_name.map(U16String::from_str);
-        let table_name = table_name.map(U16String::from_str);
-        let table_type = table_type.map(U16String::from_str);
         execute_tables(
             statement,
-            catalog_name.as_deref(),
-            schema_name.as_deref(),
-            table_name.as_deref(),
-            table_type.as_deref(),
+            &SqlText::new(catalog_name),
+            &SqlText::new(schema_name),
+            &SqlText::new(table_name),
+            &SqlText::new(table_type),
         )
     }
 
