@@ -1,12 +1,11 @@
 use std::{cmp::max, collections::HashMap, ptr::null_mut, sync::Mutex};
 
 use crate::{
-    handles::{self, log_diagnostics, OutputStringBuffer, SqlResult, State},
+    handles::{self, log_diagnostics, OutputStringBuffer, SqlResult, SqlText, State, SzBuffer},
     Connection, DriverCompleteOption, Error,
 };
 use log::debug;
 use odbc_sys::{AttrCpMatch, AttrOdbcVersion, FetchOrientation, HWnd};
-use widestring::{U16CStr, U16Str, U16String};
 
 #[cfg(target_os = "windows")]
 // Currently only windows driver manager supports prompt.
@@ -195,35 +194,13 @@ impl Environment {
         user: &str,
         pwd: &str,
     ) -> Result<Connection<'_>, Error> {
-        let data_source_name = U16String::from_str(data_source_name);
-        let user = U16String::from_str(user);
-        let pwd = U16String::from_str(pwd);
-        self.connect_utf16(&data_source_name, &user, &pwd)
-    }
+        let data_source_name = SqlText::new(data_source_name);
+        let user = SqlText::new(user);
+        let pwd = SqlText::new(pwd);
 
-    /// Allocates a connection handle and establishes connections to a driver and a data source.
-    ///
-    /// * See [Connecting with SQLConnect][1]
-    /// * See [SQLConnectFunction][2]
-    ///
-    /// # Arguments
-    ///
-    /// * `data_source_name` - Data source name. The data might be located on the same computer as
-    /// the program, or on another computer somewhere on a network.
-    /// * `user` - User identifier.
-    /// * `pwd` - Authentication string (typically the password).
-    ///
-    /// [1]: https://docs.microsoft.com/sql/odbc/reference/syntax/sqlconnect-function
-    /// [2]: https://docs.microsoft.com/sql/odbc/reference/syntax/sqlconnect-function
-    pub fn connect_utf16(
-        &self,
-        data_source_name: &U16Str,
-        user: &U16Str,
-        pwd: &U16Str,
-    ) -> Result<Connection<'_>, Error> {
         let mut connection = self.allocate_connection()?;
         connection
-            .connect(data_source_name, user, pwd)
+            .connect(&data_source_name, &user, &pwd)
             .into_result(&connection)?;
         Ok(Connection::new(connection))
     }
@@ -257,24 +234,10 @@ impl Environment {
         &self,
         connection_string: &str,
     ) -> Result<Connection<'_>, Error> {
-        let connection_string = U16String::from_str(connection_string);
-        self.connect_with_connection_string_utf16(&connection_string)
-    }
-
-    /// Allocates a connection handle and establishes connections to a driver and a data source.
-    ///
-    /// An alternative to `connect`. It supports data sources that require more connection
-    /// information than the three arguments in `connect` and data sources that are not defined in
-    /// the system information.
-    ///
-    /// To find out your connection string try: <https://www.connectionstrings.com/>
-    pub fn connect_with_connection_string_utf16(
-        &self,
-        connection_string: &U16Str,
-    ) -> Result<Connection<'_>, Error> {
+        let connection_string = SqlText::new(connection_string);
         let mut connection = self.allocate_connection()?;
         connection
-            .connect_with_connection_string(connection_string)
+            .connect_with_connection_string(&connection_string)
             .into_result(&connection)?;
         Ok(Connection::new(connection))
     }
@@ -318,7 +281,7 @@ impl Environment {
     /// let mut output_buffer = OutputStringBuffer::with_buffer_size(1024);
     /// let connection = env.driver_connect(
     ///     "",
-    ///     Some(&mut output_buffer),
+    ///     &mut output_buffer,
     ///     DriverCompleteOption::Prompt,
     /// )?;
     ///
@@ -345,7 +308,7 @@ impl Environment {
     /// let without_uid_or_pwd = "DSN=SomeSharedDatabase;";
     /// let connection = env.driver_connect(
     ///     &without_uid_or_pwd,
-    ///     Some(&mut output_buffer),
+    ///     &mut output_buffer,
     ///     DriverCompleteOption::Complete,
     /// )?;
     /// let connection_string = output_buffer.to_utf8();
@@ -368,7 +331,7 @@ impl Environment {
     /// let already_sufficient = "DSN=MicrosoftAccessFile;";
     /// let connection = env.driver_connect(
     ///    &already_sufficient,
-    ///    Some(&mut output_buffer),
+    ///    &mut output_buffer,
     ///    DriverCompleteOption::NoPrompt,
     /// )?;
     /// let connection_string = output_buffer.to_utf8();
@@ -380,7 +343,7 @@ impl Environment {
     pub fn driver_connect(
         &self,
         connection_string: &str,
-        completed_connection_string: Option<&mut OutputStringBuffer>,
+        completed_connection_string: &mut OutputStringBuffer,
         driver_completion: DriverCompleteOption,
     ) -> Result<Connection<'_>, Error> {
         #[cfg(target_os = "windows")]
@@ -428,12 +391,12 @@ impl Environment {
     pub unsafe fn driver_connect_with_hwnd(
         &self,
         connection_string: &str,
-        completed_connection_string: Option<&mut OutputStringBuffer>,
+        completed_connection_string: &mut OutputStringBuffer,
         driver_completion: DriverCompleteOption,
         parent_window: HWnd,
     ) -> Result<Connection<'_>, Error> {
         let mut connection = self.allocate_connection()?;
-        let connection_string = U16String::from_str(connection_string);
+        let connection_string = SqlText::new(connection_string);
 
         connection
             .driver_connect(
@@ -491,21 +454,23 @@ impl Environment {
             }
 
             // Allocate +1 character extra for terminating zero
-            let mut desc_buf = vec![0; desc_len as usize + 1];
-            let mut attr_buf = vec![0; attr_len as usize + 1];
+            let mut desc_buf = SzBuffer::with_capacity(desc_len as usize);
+            let mut attr_buf = SzBuffer::with_capacity(attr_len as usize);
 
             while self
                 .environment
-                .drivers_buffer_fill(FetchOrientation::Next, &mut desc_buf, &mut attr_buf)
+                .drivers_buffer_fill(
+                    FetchOrientation::Next,
+                    desc_buf.mut_buf(),
+                    attr_buf.mut_buf(),
+                )
                 .map(|res| res.into_result(&self.environment))
                 .transpose()?
                 .is_some()
             {
-                let description = U16CStr::from_slice_truncate(&desc_buf).unwrap();
-                let attributes = U16CStr::from_slice_truncate(&attr_buf).unwrap();
+                let description = desc_buf.to_utf8();
+                let attributes = attr_buf.to_utf8();
 
-                let description = description.to_string().unwrap();
-                let attributes = attributes.to_string().unwrap();
                 let attributes = attributes_iter(&attributes).collect();
 
                 driver_info.push(DriverInfo {
@@ -600,23 +565,19 @@ impl Environment {
                 driver_len = max(candidate_decs_len, driver_len);
             }
 
-            // Allocate +1 character extra for terminating zero
-            let mut server_name_buf = vec![0; server_name_len as usize + 1];
-            let mut driver_buf = vec![0; driver_len as usize + 1];
+            let mut server_name_buf = SzBuffer::with_capacity(server_name_len as usize);
+            let mut driver_buf = SzBuffer::with_capacity(driver_len as usize);
 
             let mut not_empty = self
                 .environment
-                .data_source_buffer_fill(direction, &mut server_name_buf, &mut driver_buf)
+                .data_source_buffer_fill(direction, server_name_buf.mut_buf(), driver_buf.mut_buf())
                 .map(|res| res.into_result(&self.environment))
                 .transpose()?
                 .is_some();
 
             while not_empty {
-                let server_name = U16CStr::from_slice_truncate(&server_name_buf).unwrap();
-                let driver = U16CStr::from_slice_truncate(&driver_buf).unwrap();
-
-                let server_name = server_name.to_string().unwrap();
-                let driver = driver.to_string().unwrap();
+                let server_name = server_name_buf.to_utf8();
+                let driver = driver_buf.to_utf8();
 
                 data_source_info.push(DataSourceInfo {
                     server_name,
@@ -626,8 +587,8 @@ impl Environment {
                     .environment
                     .data_source_buffer_fill(
                         FetchOrientation::Next,
-                        &mut server_name_buf,
-                        &mut driver_buf,
+                        server_name_buf.mut_buf(),
+                        driver_buf.mut_buf(),
                     )
                     .map(|res| res.into_result(&self.environment))
                     .transpose()?
@@ -683,7 +644,7 @@ fn attributes_iter(attributes: &str) -> impl Iterator<Item = (String, String)> +
 }
 
 #[cfg(test)]
-mod test {
+mod tests {
 
     use super::*;
 
