@@ -1,5 +1,7 @@
-use super::data_type::DataType;
-use std::char::{decode_utf16, DecodeUtf16Error};
+use super::{
+    data_type::DataType,
+    sql_char::{slice_to_utf8, DecodingError, SqlChar},
+};
 
 /// Indication of whether a column is nullable or not.
 #[derive(Clone, Copy, Hash, Debug, Eq, PartialEq)]
@@ -25,7 +27,7 @@ impl Nullability {
             odbc_sys::Nullability::UNKNOWN => Nullability::Unknown,
             odbc_sys::Nullability::NO_NULLS => Nullability::NoNulls,
             odbc_sys::Nullability::NULLABLE => Nullability::Nullable,
-            other => panic!("ODBC returned invalid value for Nullable: {:?}", other),
+            other => panic!("ODBC returned invalid value for Nullable: {}", other.0),
         }
     }
 }
@@ -34,7 +36,7 @@ impl Nullability {
 #[derive(Clone, Debug, Eq, PartialEq, Default)]
 pub struct ColumnDescription {
     /// Column name. May be empty if unavailable.
-    pub name: Vec<u16>,
+    pub name: Vec<SqlChar>,
     /// Type of the column
     pub data_type: DataType,
     /// Indicates whether the column is nullable or not.
@@ -44,8 +46,8 @@ pub struct ColumnDescription {
 impl ColumnDescription {
     /// Converts the internal UTF16 representation of the column name into UTF8 and returns the
     /// result as a `String`.
-    pub fn name_to_string(&self) -> Result<String, DecodeUtf16Error> {
-        decode_utf16(self.name.iter().copied()).collect()
+    pub fn name_to_string(&self) -> Result<String, DecodingError> {
+        slice_to_utf8(&self.name)
     }
 
     /// `true` if the column is `Nullable` or it is not know whether the column is nullable. `false`
@@ -57,3 +59,16 @@ impl ColumnDescription {
         }
     }
 }
+
+#[cfg(test)]
+mod tests {
+    use crate::Nullability;
+
+
+    /// Application should panic if ODBC driver returns unsupported value for nullable
+    #[test]
+    #[should_panic(expected = "ODBC returned invalid value for Nullable: 5")]
+    fn invalid_nullable_representation(){
+        Nullability::new(odbc_sys::Nullability(5));
+    }
+}
\ No newline at end of file
