@@ -4,7 +4,7 @@
 //!
 //! * `()` -> No parameter
 //! * `&a` -> Single input parameter
-//! * `&mut a` -> Input Output parameter
+//! * `InOut(&mut a)` -> Input Output parameter
 //! * `Out(&mut a)` -> Output parameter
 //! * `(a,b,c)` -> Fixed number of parameters
 //! * `&[a]` -> Arbitrary number of parameters
@@ -153,7 +153,7 @@
 //! bound as a mutable reference.
 //!
 //! ```no_run
-//! use odbc_api::{Environment, Out, Nullable};
+//! use odbc_api::{Environment, Out, InOut, Nullable};
 //!
 //! let env = Environment::new()?;
 //!
@@ -164,7 +164,7 @@
 //!
 //! conn.execute(
 //!     "{? = call TestParam(?)}",
-//!     (Out(&mut ret), &mut param))?;
+//!     (Out(&mut ret), InOut(&mut param)))?;
 //!
 //! assert_eq!(Some(99), ret.into_opt());
 //! assert_eq!(Some(7 + 5), param.into_opt());
@@ -351,16 +351,16 @@ pub unsafe trait InputParameter: HasDataType + CData {}
 /// # Safety
 ///
 /// Guarantees that there is space in the output buffer for at least one element.
-pub unsafe trait Output: CDataMut + HasDataType {}
+pub unsafe trait OutputParameter: CDataMut + HasDataType {}
 
 /// Implementers of this trait can be used as individual parameters of in a
-/// [`crate::ParameterCollection`]. They can be bound as either input parameters, output parameters
-/// or both.
-/// 
+/// [`crate::ParameterRefCollection`]. They can be bound as either input parameters, output
+/// parameters or both.
+///
 /// # Safety
 ///
 /// Parameters bound to the statement must remain valid for the lifetime of the instance.
-pub unsafe trait Parameter {
+pub unsafe trait ParameterRef {
     /// Bind the parameter in question to a specific `parameter_number`.
     ///
     /// # Safety
@@ -368,73 +368,126 @@ pub unsafe trait Parameter {
     /// Since the parameter is now bound to `stmt` callers must take care that it is ensured that
     /// the parameter remains valid while it is used. If the parameter is bound as an output
     /// parameter it must also be ensured that it is exclusively referenced by statement.
-    unsafe fn bind_parameter(
-        self,
+    unsafe fn bind_to(
+        &mut self,
         parameter_number: u16,
         stmt: &mut impl Statement,
     ) -> Result<(), Error>;
 }
 
 /// Bind immutable references as input parameters.
-unsafe impl<T: ?Sized> Parameter for &T
+unsafe impl<T: ?Sized> ParameterRef for &T
 where
     T: InputParameter,
 {
-    unsafe fn bind_parameter(
-        self,
+    unsafe fn bind_to(
+        &mut self,
         parameter_number: u16,
         stmt: &mut impl Statement,
     ) -> Result<(), Error> {
-        stmt.bind_input_parameter(parameter_number, self)
+        stmt.bind_input_parameter(parameter_number, *self)
+            .into_result(stmt)
+    }
+}
+
+/// Bind mutable references as input parameters.
+unsafe impl<T: ?Sized> ParameterRef for &mut T
+where
+    T: InputParameter,
+{
+    unsafe fn bind_to(
+        &mut self,
+        parameter_number: u16,
+        stmt: &mut impl Statement,
+    ) -> Result<(), Error> {
+        stmt.bind_input_parameter(parameter_number, *self)
+            .into_result(stmt)
+    }
+}
+
+/// Bind immutable references as input parameters.
+unsafe impl<T: ?Sized> ParameterRef for Box<T>
+where
+    T: InputParameter,
+{
+    unsafe fn bind_to(
+        &mut self,
+        parameter_number: u16,
+        stmt: &mut impl Statement,
+    ) -> Result<(), Error> {
+        stmt.bind_input_parameter(parameter_number, self.as_ref())
             .into_result(stmt)
     }
 }
 
 /// Bind mutable references as input/output parameter.
-unsafe impl<T> Parameter for &mut T
+unsafe impl<'a, T> ParameterRef for InOut<'a, T>
 where
-    T: Output,
+    T: OutputParameter,
 {
-    unsafe fn bind_parameter(
-        self,
+    unsafe fn bind_to(
+        &mut self,
         parameter_number: u16,
         stmt: &mut impl Statement,
     ) -> Result<(), Error> {
-        stmt.bind_parameter(parameter_number, odbc_sys::ParamType::InputOutput, self)
+        stmt.bind_parameter(parameter_number, odbc_sys::ParamType::InputOutput, self.0)
             .into_result(stmt)
     }
 }
 
 /// Wraps a mutable reference. Use this wrapper in order to indicate that a mutable reference should
-/// be bound as an output parameter only, rather than an input / output parameter.
+/// be bound as an input / output parameter.
 ///
 /// # Example
 ///
 /// ```no_run
-/// use odbc_api::{Environment, Out, Nullable};
+/// use odbc_api::{Environment, Out, InOut, Nullable};
 ///
 /// let env = Environment::new()?;
 ///
 /// let mut conn = env.connect("YourDatabase", "SA", "<YourStrong@Passw0rd>")?;
 ///
 /// let mut ret = Nullable::<i32>::null();
-/// let mut param = Nullable::<i32>::new(7);
+/// let mut param = Nullable::new(7);
 ///
 /// conn.execute(
 ///     "{? = call TestParam(?)}",
-///     (Out(&mut ret), &mut param))?;
+///     (Out(&mut ret), InOut(&mut param)))?;
+///
+/// # Ok::<(), odbc_api::Error>(())
+/// ```
+pub struct InOut<'a, T>(pub &'a mut T);
+
+/// Wraps a mutable reference. Use this wrapper in order to indicate that a mutable reference should
+/// be bound as an output parameter only.
+///
+/// # Example
+///
+/// ```no_run
+/// use odbc_api::{Environment, Out, InOut, Nullable};
+///
+/// let env = Environment::new()?;
+///
+/// let mut conn = env.connect("YourDatabase", "SA", "<YourStrong@Passw0rd>")?;
+///
+/// let mut ret = Nullable::<i32>::null();
+/// let mut param = Nullable::new(7);
+///
+/// conn.execute(
+///     "{? = call TestParam(?)}",
+///     (Out(&mut ret), InOut(&mut param)))?;
 ///
 /// # Ok::<(), odbc_api::Error>(())
 /// ```
 pub struct Out<'a, T>(pub &'a mut T);
 
 /// Mutable references wrapped in `Out` are bound as output parameters.
-unsafe impl<'a, T> Parameter for Out<'a, T>
+unsafe impl<'a, T> ParameterRef for Out<'a, T>
 where
-    T: Output,
+    T: OutputParameter,
 {
-    unsafe fn bind_parameter(
-        self,
+    unsafe fn bind_to(
+        &mut self,
         parameter_number: u16,
         stmt: &mut impl Statement,
     ) -> Result<(), Error> {
@@ -493,6 +546,19 @@ where
     }
 }
 
+unsafe impl<T> CDataMut for WithDataType<T>
+where
+    T: CDataMut,
+{
+    fn mut_indicator_ptr(&mut self) -> *mut isize {
+        self.value.mut_indicator_ptr()
+    }
+
+    fn mut_value_ptr(&mut self) -> *mut c_void {
+        self.value.mut_value_ptr()
+    }
+}
+
 impl<T> HasDataType for WithDataType<T>
 where
     T: HasDataType,
@@ -530,3 +596,10 @@ impl HasDataType for Box<dyn InputParameter> {
 }
 
 unsafe impl InputParameter for Box<dyn InputParameter> {}
+
+/// # Safety
+///
+/// A subclass of CData those value pointer or indicator pointer can not be changed through a
+/// mutable reference. The values these pointers point to, may change however. Used to determine,
+/// that the value has not to be rebound between multiple calls to execute.
+pub unsafe trait StableCData: CData {}
