@@ -1,4 +1,4 @@
-use crate::{handles::Statement, Error, InputParameter, Parameter};
+use crate::{handles::Statement, parameter::InputParameter, Error, ParameterRef};
 
 mod tuple;
 
@@ -60,14 +60,14 @@ mod tuple;
 /// }
 /// # Ok::<(), odbc_api::Error>(())
 /// ```
-/// 
+///
 /// # Safety
-/// 
+///
 /// Instances of this type are passed by value, so this type can be implemented by both constant and
 /// mutabale references. Implementers should take care that the values bound by `bind_parameters_to`
 /// to the statement live at least for the Duration of `self`. The most straight forward way of
 /// achieving this is of course, to bind members.
-pub unsafe trait ParameterCollection {
+pub unsafe trait ParameterRefCollection {
     /// Number of values per parameter in the collection. This can be different from the maximum
     /// batch size a buffer may be able to hold. Returning `0` will cause the the query not to be
     /// executed.
@@ -78,23 +78,23 @@ pub unsafe trait ParameterCollection {
     /// On execution a statement may want to read/write to the bound paramaters. It is the callers
     /// responsibility that by then the buffers are either unbound from the statement or still
     /// valild.
-    unsafe fn bind_parameters_to(self, stmt: &mut impl Statement) -> Result<(), Error>;
+    unsafe fn bind_parameters_to(&mut self, stmt: &mut impl Statement) -> Result<(), Error>;
 }
 
-unsafe impl<T> ParameterCollection for T
+unsafe impl<T> ParameterRefCollection for T
 where
-    T: Parameter,
+    T: ParameterRef,
 {
     fn parameter_set_size(&self) -> usize {
         1
     }
 
-    unsafe fn bind_parameters_to(self, stmt: &mut impl Statement) -> Result<(), Error> {
-        self.bind_parameter(1, stmt)
+    unsafe fn bind_parameters_to(&mut self, stmt: &mut impl Statement) -> Result<(), Error> {
+        self.bind_to(1, stmt)
     }
 }
 
-unsafe impl<T> ParameterCollection for &[T]
+unsafe impl<T> ParameterRefCollection for &[T]
 where
     T: InputParameter,
 {
@@ -102,7 +102,7 @@ where
         1
     }
 
-    unsafe fn bind_parameters_to(self, stmt: &mut impl Statement) -> Result<(), Error> {
+    unsafe fn bind_parameters_to(&mut self, stmt: &mut impl Statement) -> Result<(), Error> {
         for (index, parameter) in self.iter().enumerate() {
             stmt.bind_input_parameter(index as u16 + 1, parameter)
                 .into_result(stmt)?;
